==> raphson_mp-2.1.0-py3-none-any.whl: raphson_mp/client/track.py <==
from __future__ import annotations
import asyncio
import json
from dataclasses import dataclass
from typing import Any

from aiohttp import ClientSession

from raphson_mp.common.lyrics import LyricsLine, PlainLyrics, TimeSyncedLyrics
from raphson_mp.common.track import AudioFormat
from raphson_mp.util import urlencode


@dataclass
class DownloadedTrack:
    track: Track | None
    audio: bytes
    image: bytes
    lyrics_json: str

    @property
    def lyrics(self) -> TimeSyncedLyrics | PlainLyrics | None:
        lyrics_dict = json.loads(self.lyrics_json)
        if lyrics_dict["type"] == "none":
            return None
        elif lyrics_dict["type"] == "plain":
            return PlainLyrics(lyrics_dict["source"], lyrics_dict["text"])
        elif lyrics_dict["type"] == "synced":
            text = [LyricsLine(line["start_time"], line["text"]) for line in lyrics_dict["text"]]
            return TimeSyncedLyrics(lyrics_dict["source"], text)
        else:
            raise ValueError()

    @property
    def lyrics_text(self) -> str | None:
        lyrics = self.lyrics
        if not lyrics:
            return None
        if isinstance(lyrics, TimeSyncedLyrics):
            lyrics = lyrics.to_plain()
        return lyrics.text


@dataclass
class Track:
    path: str
    display: str
    mtime: int
    duration: int
    title: str | None
    album: str | None
    album_artist: str | None
    year: int | None
    artists: list[str]
    _session: ClientSession | None = None

    @property
    def playlist(self):
        return self.path[self.path.index("/") :]

    @classmethod
    def from_dict(cls, json_data: Any, session: ClientSession | None = None):
        return cls(
            json_data["path"],
            json_data["display"],
            json_data["mtime"],
            json_data["duration"],
            json_data["title"],
            json_data["album"],
            json_data["album_artist"],
            json_data["year"],
            json_data["artists"],
            session,
        )

    async def get_audio(self, audio_format: AudioFormat) -> bytes:
        assert self._session, "track has no ClientSession"
        async with self._session.get(
            "/track/" + urlencode(self.path) + "/audio?type=" + audio_format.value
        ) as response:
            return await response.content.read()

    async def get_cover_image(self) -> bytes:
        assert self._session, "track has no ClientSession"
        async with self._session.get("/track/" + urlencode(self.path) + "/cover?quality=high") as response:
            return await response.content.read()

    async def get_lyrics_json(self) -> str:
        assert self._session, "track has no ClientSession"
        async with self._session.get("/track/" + urlencode(self.path) + "/lyrics") as response:
            return await response.text()

    async def download(self, audio_format: AudioFormat = AudioFormat.WEBM_OPUS_HIGH) -> DownloadedTrack:
        audio, image, lyrics_json = await asyncio.gather(
            self.get_audio(audio_format),
            self.get_cover_image(),
            self.get_lyrics_json(),
        )
        return DownloadedTrack(self, audio, image, lyrics_json)

    async def download_mp3(self):
        return await self.get_audio(AudioFormat.MP3_WITH_METADATA)

==> raphson_mp-2.1.1-py3-none-any.whl: raphson_mp/client/track.py <==
from __future__ import annotations
import asyncio
import json
from dataclasses import dataclass
from typing import Any

from aiohttp import ClientSession

from raphson_mp.common.lyrics import LyricsLine, PlainLyrics, TimeSyncedLyrics
from raphson_mp.common.track import AudioFormat
from raphson_mp.common.image import ImageQuality, ImageFormat
from raphson_mp.util import urlencode


@dataclass
class DownloadedTrack:
    track: Track | None
    audio: bytes
    image: bytes
    lyrics_json: str

    @property
    def lyrics(self) -> TimeSyncedLyrics | PlainLyrics | None:
        lyrics_dict = json.loads(self.lyrics_json)
        if lyrics_dict["type"] == "none":
            return None
        elif lyrics_dict["type"] == "plain":
            return PlainLyrics(lyrics_dict["source"], lyrics_dict["text"])
        elif lyrics_dict["type"] == "synced":
            text = [LyricsLine(line["start_time"], line["text"]) for line in lyrics_dict["text"]]
            return TimeSyncedLyrics(lyrics_dict["source"], text)
        else:
            raise ValueError()

    @property
    def lyrics_text(self) -> str | None:
        lyrics = self.lyrics
        if not lyrics:
            return None
        if isinstance(lyrics, TimeSyncedLyrics):
            lyrics = lyrics.to_plain()
        return lyrics.text


@dataclass
class Track:
    path: str
    display: str
    mtime: int
    duration: int
    title: str | None
    album: str | None
    album_artist: str | None
    year: int | None
    artists: list[str]
    _session: ClientSession | None = None

    @property
    def playlist(self):
        return self.path[self.path.index("/") :]

    @classmethod
    def from_dict(cls, json_data: Any, session: ClientSession | None = None):
        return cls(
            json_data["path"],
            json_data["display"],
            json_data["mtime"],
            json_data["duration"],
            json_data["title"],
            json_data["album"],
            json_data["album_artist"],
            json_data["year"],
            json_data["artists"],
            session,
        )

    async def get_audio(self, audio_format: AudioFormat) -> bytes:
        assert self._session, "track has no ClientSession"
        async with self._session.get(
            "/track/" + urlencode(self.path) + "/audio?type=" + audio_format.value
        ) as response:
            return await response.content.read()

    async def get_cover_image(
        self, quality: ImageQuality = ImageQuality.HIGH, format: ImageFormat = ImageFormat.WEBP, meme: bool = False
    ) -> bytes:
        assert self._session, "track has no ClientSession"
        async with self._session.get(
            "/track/" + urlencode(self.path) + "/cover",
            params={"quality": quality.value, "format": format.value, "meme": 1 if meme else 0},
        ) as response:
            return await response.content.read()

    async def get_lyrics_json(self) -> str:
        assert self._session, "track has no ClientSession"
        async with self._session.get("/track/" + urlencode(self.path) + "/lyrics") as response:
            return await response.text()

    async def download(self, audio_format: AudioFormat = AudioFormat.WEBM_OPUS_HIGH) -> DownloadedTrack:
        audio, image, lyrics_json = await asyncio.gather(
            self.get_audio(audio_format),
            self.get_cover_image(),
            self.get_lyrics_json(),
        )
        return DownloadedTrack(self, audio, image, lyrics_json)

    async def download_mp3(self):
        return await self.get_audio(AudioFormat.MP3_WITH_METADATA)
